@@ -2,13 +2,7 @@
 import json
 import os
 import re
-
-try:
-    # python 3.7
-    from urllib.parse import urlparse, parse_qs  # noqa
-except ImportError:
-    # python 2.7
-    from urlparse import urlparse, parse_qs  # noqa
+from urllib.parse import parse_qs, urlparse
 
 import backoff
 
@@ -171,7 +165,9 @@ def fatal_request_error(err=None):
     max_time=MAX_RETRY_TIME_SECONDS,
     giveup=fatal_request_error,
 )
-def download_file(file_path, download_url, skip_existing=True):
+def download_file(
+    file_path, download_url, skip_existing=True, no_progress=False
+):
     """Download a file to file system.
 
     Args:
@@ -179,6 +175,7 @@ def download_file(file_path, download_url, skip_existing=True):
             and download template
         download_url (str): url of the file to download
         skip_existing (bool): skip already downloaded files
+        no_progress (bool): don't show progress bar
 
     Returns:
         str : file path
@@ -191,11 +188,11 @@ def download_file(file_path, download_url, skip_existing=True):
         headers = dict(
             Range="bytes={}-".format(os.path.getsize(file_path_tmp))
         )
-        echo("Resuming previous download: {}".format(file_path))
+        echo("Resuming previous download: {}".format(file_path), err=True)
     else:
         file_mode = "wb"
         headers = dict()
-        echo("Downloading file to {}".format(file_path))
+        echo("Downloading file to {}".format(file_path), err=True)
 
     stream_params = dict(
         stream=True, allow_redirects=False, headers=headers, timeout=30
@@ -204,26 +201,29 @@ def download_file(file_path, download_url, skip_existing=True):
     with requests.get(download_url, **stream_params) as req:
         req.raise_for_status()
         total = int(req.headers["content-length"])
-        # pylint: disable=C0330
+        # pylint: disable=E0012,C0330
         if (
             skip_existing
             and os.path.isfile(file_path)
             and os.path.getsize(file_path) == total
         ):
-            echo("Skipping existing file: {}".format(file_path))
+            echo("Skipping existing file: {}".format(file_path), err=True)
             return file_path
 
         echo_debug("Starting to download file to: {}".format(file_path))
 
         with open(file_path_tmp, file_mode) as downloaded_file:
-            pbar = get_progress_bar(
-                int(req.headers["content-length"]), "Downloading: "
-            )
-            pbar.start()
+            if not no_progress:
+                pbar = get_progress_bar(
+                    int(req.headers["content-length"]), "Downloading: "
+                )
+                pbar.start()
             for chunk in req.iter_content(chunk_size=CHUNK_SIZE):
                 downloaded_file.write(chunk)
-                pbar.update(pbar.value + len(chunk))
-            pbar.finish()
+                if not no_progress:
+                    pbar.update(pbar.value + len(chunk))
+            if not no_progress:
+                pbar.finish()
 
         # Cross-platform cross-python-version file overwriting
         if os.path.exists(file_path):
@@ -233,10 +233,26 @@ def download_file(file_path, download_url, skip_existing=True):
             )
             os.remove(file_path)
         os.rename(file_path_tmp, file_path)
-        echo("Finished downloading a file: {}".format(file_path))
+        echo("Finished downloading a file: {}".format(file_path), err=True)
         return file_path
 
 
+def save_metadata_file(path, content):
+    """Helper function to save metadata to json file.
+
+    Args:
+        path(str): full file path where metadata will be saved
+        content(any): depending on the metadata, any json serializable
+
+    Returns:
+        None
+    """
+    echo("Downloading file to: {}".format(path), err=True)
+    with open(path, "w") as metadata_file:
+        json.dump(content, metadata_file)
+    echo("Finished downloading a file: {}".format(path), err=True)
+
+
 def save_qc_file(path, content):
     """Helper function to save qc metrics to json file.
 
@@ -247,9 +263,10 @@ def save_qc_file(path, content):
     Returns:
         None
     """
-    echo("Downloading file to: {}".format(path))
+    echo("Downloading file to: {}".format(path), err=True)
     with open(path, "w") as qc_file:
         json.dump(content, qc_file)
+    echo("Finished downloading a file: {}".format(path), err=True)
 
 
 def fatal_process_sample_error(err):
