@@ -10,7 +10,9 @@ from .main import ListSamples
 
 @click.command("list-samples")
 @click.argument("project_id")
-@click.option("--search", help="Gencove sample ID or client ID")
+@click.option(
+    "--search", help="Gencove sample ID, client ID or metadata substring"
+)
 @click.option(
     "--status",
     help="Get samples with specific status",
@@ -18,7 +20,7 @@ from .main import ListSamples
     default=SAMPLE_STATUS.all,
 )
 @add_options(common_options)
-def list_project_samples(  # pylint: disable=C0330,R0913
+def list_project_samples(  # pylint: disable=E0012,C0330,R0913
     project_id, search, status, host, email, password, api_key
 ):
     """List samples in a project."""
