@@ -17,6 +17,11 @@ _APIEndpoint = namedtuple(
         "sample_sheet",
         "projects",
         "pipeline_capabilities",
+        "project_batch_types",
+        "project_batches",
+        "batches",
+        "project_merge_vcfs",
+        "sample_metadata",
     ],
 )
 API_ENDPOINTS = _APIEndpoint(
@@ -31,6 +36,11 @@ API_ENDPOINTS = _APIEndpoint(
     "/api/v2/sample-sheet/",
     "/api/v2/projects/",
     "/api/v2/pipeline-capabilities/{id}",
+    "/api/v2/project-batch-types/{id}",
+    "/api/v2/project-batches/{id}",
+    "/api/v2/batches/{id}",
+    "/api/v2/project-merge-vcfs/{id}",
+    "/api/v2/sample-metadata/{id}",
 )
 
 _SampleAssignmentStatus = namedtuple(
